import os
import json


class TestMetadata:
    """
    Reads and stores metadata about the current test run.
    For example:
    - Browser type (chromium, firefox, webkit)
    - Viewport size
    - Device emulation
    - Base URL
    - Credentials
    """

    def __init__(self, base_path: str = ".", config_file: str = "bugster.json"):
        self.base_path = base_path
        self.config_file = os.path.join(base_path, config_file)
        self.config = {}
        if os.path.exists(self.config_file):
            with open(self.config_file, "r") as f:
                self.config = json.load(f)

    def get_browser(self, default: str = "chromium") -> str:
        return self.config.get("settings", {}).get("browser", default)

    def get_headless(self, default: bool = True) -> str:
        return self.config.get("settings", {}).get("headless", default)

    def get_viewport(self) -> dict:
        return self.config.get("settings", {}).get(
            "viewport", {"width": 1920, "height": 1080}
        )

    def get_credentials(self) -> dict:
        """
        Example: loads credentials from config or environment
        """
        creds = self.config.get("auth", {}).get("credentials", [])
        creds = {
            "email": creds[0].get("email", "test@example.com"),
            "password": creds[0].get("password", "secret"),
        }
        return creds

    def get_base_url(self) -> str:
        return (
            self.config.get("environment", {})
            .get("dev", {})
            .get("base_url", "https://www.example.com")
        )

    def get_auth_url(self) -> str:
        return (
            self.config.get("environment", {})
            .get("dev", {})
            .get("auth_url", "https://www.example.com")
        )

    def get_login_strategy(self):
        return self.config.get("auth", {}).get(
            "strategy", "default.path:BaseLoginStrategy"
        )

    def get_login_instructions(self):
        return self.config.get("auth", {}).get("instructions", {})

    def get_screenshot_path(self):
        return os.path.join(self.base_path, "screenshots")

    def get_trace_path(self):
        return os.path.join(self.base_path, "traces")
